#ifndef _RAID1_H
#define _RAID1_H

#include <linux/raid/md.h>

typedef struct r1bio_s r1bio_t;

struct r1_private_data_s {
	mddev_t			*mddev;
	int			raid_disks;
	mdk_member_t		*last_used;
	sector_t		next_seq_sect;

	mdk_sync_support_t	syncinfo;

	mempool_t *r1bio_pool;
	mempool_t *r1buf_pool;
};

typedef struct r1_private_data_s conf_t;

#define mddev_to_conf(mddev) ((conf_t *) mddev->member.pers_private)

struct r1_member_private_data_s {
	sector_t	head_position;
	int		sync_member;
};

typedef struct r1_member_private_data_s mconf_t;

#define member_to_mconf(member) ((mconf_t *) (member)->parent_pers_private)

struct r1bio_holder_s {
	mdk_member_t	*member;
	r1bio_t		*r1bio;
	struct bio	*bio;
};
typedef struct r1bio_holder_s r1bio_holder_t;

/*
 * this is our 'private' RAID1 bio.
 *
 * it contains information about what kind of IO operations were started
 * for this RAID1 operation, and about their status:
 */

struct r1bio_s {
	atomic_t		remaining; /* 'have we finished' count,
					    * used from IRQ handlers
					    */
	int			phase;
	sector_t		sector;
	int			sectors;
	unsigned long		state;
	mddev_t			*mddev;
	/*
	 * original bio going to /dev/mdx
	*/
	struct bio              *master_bio;
	struct list_head	retry_list;

	/*
	 * The array's number of configured raid disks may change
	 * while this r1bio is in flight.  Avoid the race by keeping
	 * a static value that indicates the size of our bio array.
	 */
	u_int			num_bios;

	/*
	 * if the IO is in WRITE direction, then multiple bios are used.
	 * We choose the number when they are allocated.
	 */
	r1bio_holder_t		bios[0];
};

/* bits for r1bio.state */
#define	R1BIO_Uptodate	0
#define	R1BIO_IsSync	1
#endif
